@@ -1,5 +1,8 @@
 #!/usr/bin/env python
-__version__ = '$Revision: 1.2 $ $Date: 2003-11-16 14:50:52+01 $ kgm'
+"""Lister 1.4.2
+Pretty-printer for SimPy class objects
+__version__ = '$Revision: 1.1 $ $Date: 2004-05-19 20:15:22+02 $ kgm'
+"""
 class Lister(object):
     indent=0
     def __str__(self):
