@@ -1,7 +1,7 @@
 #!/usr/bin/env python
-""" SimPlot 1.4alpha, December 20, 2003
+""" SimPlot 1.4.2
 
-$Revision: 1.20 $ $Date: 2003-12-20 17:37:21+01 $ kgm
+$Revision: 1.1 $ $Date: 2004-05-19 20:27:23+02 $ kgm
 
 LICENSE:
 Copyright (C) 2002  Klaus G. Muller, Tony Vignaux
@@ -35,6 +35,7 @@ Change history:
     Nov 2, 2003 :  Combined utils.py (also from Grayson) with plotting package.
     Nov 11, 2003: Made totally OO
     Dec 16, 2003: Completion of SimPlot 1.4alpha
+    Feb 2004:       Release with SimPy 1.4
 
 """
 from Tkinter import *
