@@ -1,6 +1,6 @@
 #!/usr/bin/env python
-"""Simulation 1.4alpha
-__version__ = '$Revision: 1.7 $ $Date: 2003-12-13 12:26:08+01 $ kgm'
+"""Simulation 1.4.2
+__version__ = '$Revision: 1.1 $ $Date: 2004-05-19 20:28:22+02 $ kgm'
 LICENSE:
 Copyright (C) 2002  Klaus G. Muller, Tony Vignaux
 mailto: kgmuller@xs4all.nl and Tony.Vignaux@vuw.ac.nz
@@ -24,8 +24,6 @@ Implements SimPy Processes, resources, and the backbone simulation scheduling
 by coroutine calls.
 Based on generators (Python 2.2 and later)
 
-
-
 **Change history:**
 
     Started out as SiPy 0.9
@@ -33,7 +31,7 @@ Based on generators (Python 2.2 and later)
     5/9/2002: SiPy 0.9.1
     
         - Addition of '_cancel' method in class Process and supporting '_unpost' method in 
-	  class __Evlist.
+          class __Evlist.
         
         - Removal of redundant 'Action' method in class Process.
         
@@ -195,6 +193,9 @@ Based on generators (Python 2.2 and later)
     Oct 2003: Added monitored Resource instances (Monitors for activeQ and waitQ)
 
     13 Dec 2003: Merged in Monitor and Histogram
+
+    27 Feb 2004: Repaired bug in activeQ monitor of class Resource. Now actMon
+                 correctly records departures from activeQ.
   
 """
 from __future__ import generators
@@ -202,7 +203,7 @@ from SimPy.Lister import *
 import bisect
 import types
 __TESTING=False
-__version__="1.4alpha 11/11/03"
+__version__="1.4.1 27 Feb 2004"
 if __TESTING: print "SimPy.Simulation %s" %__version__
 hold=1234
 passivate=5678
@@ -486,8 +487,9 @@ class Resource(Lister):
         self.capacity=capacity  # resource units in this resource
         self.unitName=unitName  # type name of resource units
         self.n=capacity         # uncommitted resource units
+        self.monitored=monitored
 
-        if monitored:           # Monitor waitQ, activeQ
+        if self.monitored:           # Monitor waitQ, activeQ
             self.actMon=Monitor(name="Active Queue Monitor %s"%self.name,
                                  ylab="nr in queue",tlab="time")
             monact=self.actMon
@@ -501,8 +503,10 @@ class Resource(Lister):
         self.preemptable=preemptable
         self.activeQ=qType(self,monact)
         self.priority_default=0
+        self.count=0
         
     def _request(self,arg):
+        self.count+=1
         """Process request event for this resource"""
         object=arg[1].who
         if len(arg[0]) == 4:        # yield request,self,resource,priority
@@ -549,9 +553,12 @@ class Resource(Lister):
                 at=_t,prior=1)
 
     def _release(self,arg):
+        self.count-=1
         """Process release request for this resource"""
         self.n += 1
         self.activeQ.remove(arg[1].who)
+        if self.monitored:
+            self.actMon.observe(len(self.activeQ),t=now())
         #reactivate first waiting requestor if any; assign Resource to it
         if self.waitQ:
             object=self.waitQ.leave()
