@@ -1,6 +1,6 @@
 #!/usr/bin/env python
-"""SimulationRT 1.4alpha
-__version__ = '$Revision: 1.4 $ $Date: 2003-12-13 12:22:29+01 $ kgm'
+"""SimulationRT 1.4.2
+__version__ = '$Revision: 1.2 $ $Date: 2004-05-19 20:31:09+02 $ kgm'
 LICENSE:
 Copyright (C) 2002  Klaus G. Muller, Tony Vignaux
 mailto: kgmuller@xs4all.nl and Tony.Vignaux@vuw.ac.nz
@@ -36,6 +36,12 @@ Based on generators (Python 2.2 and later)
             - Updated to Simulation 1.4alpha API
 
     13 Dec 2003: Merged in Monitor and Histogram
+
+    27 Feb 2004: Repaired bug in activeQ monitor of class Resource. Now actMon
+                 correctly records departures from activeQ.
+
+    May 19, 2004: added missing Histogram class	
+		
 """
 from __future__ import generators
 from SimPy.Lister import *
@@ -43,8 +49,8 @@ import bisect
 import types
 import time
 __TESTING=False
-__version__="1.4alpha 11/12/03"
-if __TESTING: print "SimPy.Simulation %s" %__version__
+__version__="1.4.1 27 Feb 2004"
+if __TESTING: print "SimPy.SimulationRT %s" %__version__
 hold=1234
 passivate=5678
 request=135
@@ -336,8 +342,9 @@ class Resource(Lister):
         self.capacity=capacity  # resource units in this resource
         self.unitName=unitName  # type name of resource units
         self.n=capacity         # uncommitted resource units
+        self.monitored=monitored
 
-        if monitored:           # Monitor waitQ, activeQ
+        if self.monitored:           # Monitor waitQ, activeQ
             self.actMon=Monitor(name="Active Queue Monitor %s"%self.name,
                                  ylab="nr in queue",tlab="time")
             monact=self.actMon
@@ -402,6 +409,8 @@ class Resource(Lister):
         """Process release request for this resource"""
         self.n += 1
         self.activeQ.remove(arg[1].who)
+        if self.monitored:
+            self.actMon.observe(len(self.activeQ),now())
         #reactivate first waiting requestor if any; assign Resource to it
         if self.waitQ:
             object=self.waitQ.leave()
@@ -492,6 +501,29 @@ def simulate(until=0,real_time=False,rel_speed=1):
             _stop = True
     _e=None
     return message
+    
+class Histogram(list):
+    """ A histogram gathering and sampling class"""
+
+    def __init__(self,name = '',low=0.0,high=100.0,nbins=10):
+        list.__init__(self)
+        self.name  = name
+        self.low   = low
+        self.high  = high
+        self.nbins = nbins
+        self.binsize=(self.high-self.low)/nbins
+        #self[:] = [[1,2],[3,4]]
+        self[:] =[[low+(i-1)*self.binsize,0] for i in range(self.nbins+2)]
+        #print '__init__ :', self[0],self
+       
+    def addIn(self,y):
+        """ add a value into the correct bin"""
+        b = int((y-self.low+self.binsize)/self.binsize)
+        if b < 0: b = 0
+        if b > self.nbins+1: b = self.nbins+1
+        assert 0 <= b <=self.nbins+1,'Histogram.addIn: b out of range: %s'%b
+        self[b][1]+=1
+
 
 class Monitor(list):
     """ Monitored variables
