@@ -1,6 +1,6 @@
 #!/usr/bin/env python
-"""Simulation 1.4alpha
-__version__ = '$Revision: 1.3 $ $Date: 2003-12-13 12:22:58+01 $ kgm'
+"""SimulationStep 1.4.2
+__version__ = '$Revision: 1.2 $ $Date: 2004-05-20 14:08:05+02 $ kgm'
 LICENSE:
 Copyright (C) 2002  Klaus G. Muller, Tony Vignaux
 mailto: kgmuller@xs4all.nl and Tony.Vignaux@vuw.ac.nz
@@ -196,7 +196,12 @@ Based on generators (Python 2.2 and later)
 
     November 2003: Brought up to Simulation 1.4alpha
 
-    13 Dec 2003: Merged in Monitor and Histogram    
+    13 Dec 2003: Merged in Monitor and Histogram
+
+    27 Feb 2004: Repaired bug in activeQ monitor of class Resource. Now actMon
+                 correctly records departures from activeQ.
+		 
+    May 19, 2004: Added erroneously omitted Histogram class.
 """
 
 
@@ -205,8 +210,8 @@ from SimPy.Lister import *
 import bisect
 import types
 __TESTING=True
-__version__="1.4alpha 11/11/03"
-if __TESTING: print "SimPy.Simulation %s" %__version__
+__version__="1.4.2 19 May 2004"
+if __TESTING: print "SimPy.SimulationStep %s" %__version__
 hold=1234
 passivate=5678
 request=135
@@ -282,7 +287,8 @@ class Process(Lister):
            
     def cancel(self,victim): 
         """Application function to cancel all event notices for this Process
-	instance;(should be all event notices for the _generator_)."""
+           instance;(should be all event notices for the _generator_)."""
+        
         _e._unpost(whom=victim)
         
     def _hold(self,a):
@@ -327,8 +333,8 @@ def showEvents():
 class __Evlist:
     """Defines event list and operations on it"""
     def __init__(self):
-        #has the structure {<time1>:(ev notice1, ev notice2, . . ),
-	#                   <time2>:(ev notice 3,...),..}
+        #Event list has the structure {<time1>:(ev notice1, ev notice2, . . ),
+        #                              <time2>:(ev notice 3,...),..}
         self.events={}
         #always sorted list of event times (keys for self.events)	
         self.timestamps=[]
@@ -340,7 +346,7 @@ class __Evlist:
             raise SimError("Attempt to schedule event in the past")
         if at in self.events:
             if prior:
-	        #before all other event notices at this time
+            #before all other event notices at this time
                 self.events[at][0:0]= [what] 
             else:
                 self.events[at].append(what)
@@ -364,11 +370,11 @@ class __Evlist:
                     self.events[thistime].remove(n)
                     whom._nextTime = None 
                     if not self.events[thistime]:
-			# no other ev notices for thistime
+                    # no other ev notices for thistime
                         del(self.events[thistime])
                         item_delete_point = bisect.bisect(self.timestamps,
-			                                  thistime)
-                        del self.timestamps[item_delete_point-1]	          
+                                              thistime)
+                        del self.timestamps[item_delete_point-1]
                                               
     def _nextev(self):
         """Retrieve next event from event list"""
@@ -412,7 +418,7 @@ def activate(object,process,at="undefined",delay="undefined",prior=False):
                        " is not a generator (contains no 'yield')")
     if not object._terminated and not object._nextTime: 
         #store generator reference in object; needed for reactivation
-	object._nextpoint=process       
+        object._nextpoint=process       
         if at=="undefined":
             at=_t
         if delay=="undefined":
@@ -436,12 +442,17 @@ def reactivate(object,at="undefined",delay="undefined",prior=False):
         else:
             zeit=max(_t,_t+delay)
         _e._post(_Action(who=object,generator=object._nextpoint),at=zeit,
-	         prior=prior)
+             prior=prior)
 
 
     
 class Queue(list):
-    def __init__(self,res):
+    def __init__(self,res,moni):
+        if moni==[]:
+            self.monit=True # True if Monitor
+        else:
+            self.monit=False
+        self.moni=moni # The Monitor
         self.resource=res # the resource this queue belongs to
 
     def enter(self,object):
@@ -451,25 +462,30 @@ class Queue(list):
         pass
 
 class FIFO(Queue):
-    def __init__(self,res):
-        Queue.__init__(self,res)
+    def __init__(self,res,moni):
+        Queue.__init__(self,res,moni)
 
     def enter(self,object):
         self.append(object)
+        if self.monit:
+            self.moni.observe(len(self),t=now())
 
     def leave(self):
-        return self.pop(0)
+        a= self.pop(0)
+        if self.monit:
+            self.moni.observe(len(self),t=now())
+        return a
 
 class PriorityQ(FIFO):
     """Queue is always ordered according to priority.
     Higher value of priority attribute == higher priority.
     """
-    def __init__(self,res):
-        FIFO.__init__(self,res)
+    def __init__(self,res,moni):
+        FIFO.__init__(self,res,moni)
 
     def enter(self,object):
         if len(self):
-	    ix=self.resource
+            ix=self.resource
             if self[-1]._priority[ix] >= object._priority[ix]:
                 self.append(object)
             else:
@@ -479,25 +495,40 @@ class PriorityQ(FIFO):
                 self.insert(z,object)
         else:
             self.append(object)
+        if self.monit:
+            self.moni.observe(len(self),t=now())
 
-class Resource:
+class Resource(Lister):
     """Models shared, limited capacity resources with queuing;
     FIFO is default queuing discipline.
     """
     
     def __init__(self,capacity=1,name="a_resource",unitName="units",
-                 qType=FIFO,preemptable=0): 
-        self.name=name
-        self.capacity=capacity
-        self.unitName=unitName
-        self.n=capacity
+                 qType=FIFO,preemptable=0,monitored=False): 
+        self.name=name          # resource name
+        self.capacity=capacity  # resource units in this resource
+        self.unitName=unitName  # type name of resource units
+        self.n=capacity         # uncommitted resource units
+        self.monitored=monitored
 
-        self.waitQ=qType(self)
+        if self.monitored:           # Monitor waitQ, activeQ
+            self.actMon=Monitor(name="Active Queue Monitor %s"%self.name,
+                                 ylab="nr in queue",tlab="time")
+            monact=self.actMon
+            self.waitMon=Monitor(name="Wait Queue Monitor %s"%self.name,
+                                 ylab="nr in queue",tlab="time")
+            monwait=self.waitMon
+        else:
+            monwait=None
+            monact=None            
+        self.waitQ=qType(self,monwait)
         self.preemptable=preemptable
-        self.activeQ=qType(self)
+        self.activeQ=qType(self,monact)
         self.priority_default=0
+        self.count=0
         
     def _request(self,arg):
+        self.count+=1
         """Process request event for this resource"""
         object=arg[1].who
         if len(arg[0]) == 4:        # yield request,self,resource,priority
@@ -527,7 +558,7 @@ class Resource:
                 self.activeQ.enter(object)                    
                 # post event notice for preempting process
                 _e._post(_Action(who=object,generator=object._nextpoint),
-		         at=_t,prior=1)
+                         at=_t,prior=1)
             else:
                 self.waitQ.enter(object)
                 # passivate
@@ -541,12 +572,15 @@ class Resource:
                 self.n -= 1
                 self.activeQ.enter(object)
                 _e._post(_Action(who=object,generator=object._nextpoint),
-		         at=_t,prior=1)
+                at=_t,prior=1)
 
     def _release(self,arg):
+        self.count-=1
         """Process release request for this resource"""
         self.n += 1
         self.activeQ.remove(arg[1].who)
+        if self.monitored:
+            self.actMon.observe(len(self.activeQ),t=now())
         #reactivate first waiting requestor if any; assign Resource to it
         if self.waitQ:
             object=self.waitQ.leave()
@@ -566,11 +600,7 @@ class Resource:
             else:
                 reactivate(object,delay=0,prior=1)
         _e._post(_Action(who=arg[1].who,generator=arg[1].who._nextpoint),
-		  at=_t,prior=1)
-
-
-    def __str__(self):
-        return "<Resource %s, %d %s>" %(self.name,self.n,self.unitName)
+                at=_t,prior=1)
 
 def scheduler(till=0):
     """Schedules Processes/semi-coroutines until time 'till'.
@@ -680,7 +710,28 @@ def simulateStep(callback=lambda :None,until=0):
         if _step:
             callback()
     return message,status
+    
+class Histogram(list):
+    """ A histogram gathering and sampling class"""
 
+    def __init__(self,name = '',low=0.0,high=100.0,nbins=10):
+        list.__init__(self)
+        self.name  = name
+        self.low   = low
+        self.high  = high
+        self.nbins = nbins
+        self.binsize=(self.high-self.low)/nbins
+        #self[:] = [[1,2],[3,4]]
+        self[:] =[[low+(i-1)*self.binsize,0] for i in range(self.nbins+2)]
+        #print '__init__ :', self[0],self
+       
+    def addIn(self,y):
+        """ add a value into the correct bin"""
+        b = int((y-self.low+self.binsize)/self.binsize)
+        if b < 0: b = 0
+        if b > self.nbins+1: b = self.nbins+1
+        assert 0 <= b <=self.nbins+1,'Histogram.addIn: b out of range: %s'%b
+        self[b][1]+=1
 
 class Monitor(list):
     """ Monitored variables
@@ -817,21 +868,21 @@ def test_demo():
             for i in range(1):
                 Aa.sequIn.append(self.name)
                 print now(),rrr,"waitQ:",len(rrr.waitQ),"activeQ:",\
-		      len(rrr.activeQ)
+                      len(rrr.activeQ)
                 print "waitQ: ",[(k.name,k._priority[rrr]) for k in rrr.waitQ]
                 print "activeQ: ",[(k.name,k._priority[rrr]) \
-		                   for k in rrr.activeQ]
+                           for k in rrr.activeQ]
                 assert rrr.n+len(rrr.activeQ)==rrr.capacity, \
-		       "Inconsistent resource unit numbers"
+                       "Inconsistent resource unit numbers"
                 print now(),self.name,"requests 1 ", rrr.unitName
                 yield request,self,rrr,priority
                 print now(),self.name,"has 1 ",rrr.unitName
                 print now(),rrr,"waitQ:",len(rrr.waitQ),"activeQ:",\
-		      len(rrr.activeQ)
+                      len(rrr.activeQ)
                 print now(),rrr,"waitQ:",len(rrr.waitQ),"activeQ:",\
-		      len(rrr.activeQ)
+                      len(rrr.activeQ)
                 assert rrr.n+len(rrr.activeQ)==rrr.capacity, \
-		       "Inconsistent resource unit numbers"
+                       "Inconsistent resource unit numbers"
                 yield hold,self,self.holdtime
                 print now(),self.name,"gives up 1",rrr.unitName
                 yield release,self,rrr
@@ -839,9 +890,9 @@ def test_demo():
                 print now(),self.name,"has released 1 ",rrr.unitName
                 print "waitQ: ",[(k.name,k._priority[rrr]) for k in rrr.waitQ]
                 print now(),rrr,"waitQ:",len(rrr.waitQ),"activeQ:",\
-		      len(rrr.activeQ)
+                      len(rrr.activeQ)
                 assert rrr.n+len(rrr.activeQ)==rrr.capacity, \
-		       "Inconsistent resource unit numbers"
+                       "Inconsistent resource unit numbers"
 
     class Destroyer(Process):
         def __init__(self):
