@@ -1,6 +1,6 @@
 #!/usr/bin/env python
-"""SimulationTrace 1.4alpha 11 December 2003
-__version__ = '$Revision: 1.5 $ $Date: 2003-12-13 12:23:52+01 $ kgm'
+"""SimulationTrace 1.4.2
+__version__ = '$Revision: 1.3 $ $Date: 2004-05-20 14:10:19+02 $ kgm'
 LICENSE:
 Copyright (C) 2002  Klaus G. Muller, Tony Vignaux
 mailto: kgmuller@xs4all.nl and Tony.Vignaux@vuw.ac.nz
@@ -31,6 +31,13 @@ Based on generators (Python 2.2 and later)
     November 03: Brought up to Simulation 1.4alpha
     
     13 Dec 2003: Merged in Monitor and Histogram
+
+    31 Mar 2004: Additional tracing of preempts; repair of small
+                 bug in recordCancel
+		 
+    15 May 2005: Repaired function thold()
+    
+    19 May 2004: Added erroneously omitted Histogram class.
 """
 
 
@@ -40,7 +47,7 @@ import bisect
 import types
 import sys
 __TESTING=False
-__version__="1.3 June 22 2003"
+__version__="1.4.2 19 May 2004"
 if __TESTING: print "SimPy.SimulationTrace %s" %__version__
 hold=1234
 passivate=5678
@@ -241,8 +248,7 @@ def activate(object,process,at="undefined",delay="undefined",prior=False):
         else:
             zeit=max(_t,_t+delay)
         _e._post(_Action(who=object,generator=process),at=zeit,prior=prior)
-	trace.recordActivate(object,zeit,prior)
-
+    trace.recordActivate(object,zeit,prior)
 
 def reactivate(object,at="undefined",delay="undefined",prior=False):
     """Application function to reactivate a process which is active,
@@ -322,8 +328,9 @@ class Resource(Lister):
         self.capacity=capacity  # resource units in this resource
         self.unitName=unitName  # type name of resource units
         self.n=capacity         # uncommitted resource units
+        self.monitored=monitored
 
-        if monitored:           # Monitor waitQ, activeQ
+        if self.monitored:           # Monitor waitQ, activeQ
             self.actMon=Monitor(name="Active Queue Monitor %s"%self.name,
                                  ylab="nr in queue",tlab="time")
             monact=self.actMon
@@ -386,6 +393,8 @@ class Resource(Lister):
         """Process release request for this resource"""
         self.n += 1
         self.activeQ.remove(arg[1].who)
+        if self.monitored:
+            self.actMon.observe(len(self.activeQ),t=now())
         #reactivate first waiting requestor if any; assign Resource to it
         if self.waitQ:
             object=self.waitQ.leave()
@@ -519,7 +528,10 @@ class Trace(Lister):
             return True
 
     def thold(self,par):
-        return "delay: %s"%par[0][2]
+	try:
+	    return "delay: %s"%par[0][2]
+	except:
+		return 0
     thold=classmethod(thold)
     
     def trequest(self,par):
@@ -565,7 +577,7 @@ class Trace(Lister):
         self._comment=None
                 
     def recordCancel(self,who,victim):
-        if self.ifTrace("interrupt" in self.toTrace):
+        if self.ifTrace("cancel" in self.toTrace):
             print >>self.outfile,"%s cancel by: <%s> of: <%s>"%(now(),who.name,victim.name)
             if self._comment:
                 print >>self.outfile,"----",self._comment
@@ -596,6 +608,29 @@ class Trace(Lister):
 
     def ttext(self,par):
         self._comment=par
+        
+class Histogram(list):
+    """ A histogram gathering and sampling class"""
+
+    def __init__(self,name = '',low=0.0,high=100.0,nbins=10):
+        list.__init__(self)
+        self.name  = name
+        self.low   = low
+        self.high  = high
+        self.nbins = nbins
+        self.binsize=(self.high-self.low)/nbins
+        #self[:] = [[1,2],[3,4]]
+        self[:] =[[low+(i-1)*self.binsize,0] for i in range(self.nbins+2)]
+        #print '__init__ :', self[0],self
+       
+    def addIn(self,y):
+        """ add a value into the correct bin"""
+        b = int((y-self.low+self.binsize)/self.binsize)
+        if b < 0: b = 0
+        if b > self.nbins+1: b = self.nbins+1
+        assert 0 <= b <=self.nbins+1,'Histogram.addIn: b out of range: %s'%b
+        self[b][1]+=1
+
   
 class Monitor(list):
     """ Monitored variables
