@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # testSimPy.py test routines
 #        $Author: klaus $              $Revision: 1.0 $
-#        $Date: 2003-10-25 14:52:21+02 $
+#        $Date: 2004-05-15 11:09:04+02 $
 # 2002 11 15 Added tests for priority queues and preemption
 # 2002 11 22 testing problem in accum
 # 2003 03 30 added tests for SEP001v17 interrupts
@@ -9,10 +9,12 @@
 # 2003 04 08 added tests for process state transitions
 # 2003 04 10 changed to "self.cancel(victim)" syntax
 # 2003 04 13 removed dummy init assertions
+# 2004 02 28 added test for monitored queues (gav)
+# 2004 05 03 corrected test for monitored queues (gav)
 
 """Testing SimPy
 """
-__version__ = '$Revision: 1.0 $ $Date: 2003-10-25 14:52:21+02 $ kgm'
+__version__ = '$Revision: 1.0 $ $Date: 2004-05-15 11:09:04+02 $ kgm'
 Revision = '$Revision: 1.0 $'
 
 from __future__ import generators
@@ -318,6 +320,32 @@ class makeResourcetestcase(unittest.TestCase):
        assert [x[1] for x in nuJob.seqOut] == [start3+n3.serviceTime,start2+2*n2.serviceTime,90], "Wrong service sequence/times: "+str([x for x in nuJob.seqOut])
 
 
+   def testmonitored(self):
+      """ test monitoring of number in the two queues, waitQ and activeQ
+      """
+      class Job(Process):
+          def __init__(self,name):
+             Process.__init__(self,name)
+             
+          def execute(self,res):
+             yield request,self,res
+             yield hold,self,2
+             yield release,self,res
+             
+      initialize()
+      res=Resource(name="server",capacity=1,monitored=1)
+      n1=Job(name="Job 1")
+      n2=Job(name="Job 2")
+      n3=Job(name="Job 3")
+      activate(n1,n1.execute(res),at=2)
+      activate(n2,n2.execute(res),at=2)
+      activate(n3,n3.execute(res),at=2) # 3 arrive at 2
+      simulate(until=100)
+      assert res.waitMon == [[2, 1], [2, 2], [4, 1], [6, 0]],'Wrong waitMon:%s'%res.waitMon
+      assert res.actMon == [[2, 1], [4, 0], [4, 1], [6, 0], [6, 1], [8, 0]],'Wrong actMon:%s'%res.actMon
+      #print res.actMon
+      assert res.waitMon.timeAverage() == (0*2+2*2+1*2)/8.0,'Wrong waitMon.timeAverage:%s'%res.waitMon.timeAverage()
+      
                
 def makeRSuite():
     suite = unittest.TestSuite()
@@ -330,116 +358,13 @@ def makeRSuite():
     testrequestPriority1 = makeResourcetestcase("testrequestPriority1")
     testrequestPriority2 = makeResourcetestcase("testrequestPriority2")
     testrequestPriority3 = makeResourcetestcase("testrequestPriority3")
+    testmonitored = makeResourcetestcase("testmonitored")
     suite.addTests([testInit,testrequest,testrequest2,testrequest3,testrequest4,testrequestPriority,
-                    testrequestPriority1,testrequestPriority2,testrequestPriority3]) 
+                    testrequestPriority1,testrequestPriority2,testrequestPriority3,
+                    testmonitored]) 
     return suite
       
 
-## ------------------------------------------------------------
-##class Thing(Process):
-##   """ Thing class for testing Monitors in simulation"""
-##   def __init__(self,M=None,name=""):
-##        Process.__init__(self)
-##        self.name=name
-##        self.x = 0.0
-##        self.M = M
-##
-##   def execute(self):       
-##        yield hold,self,10.0
-##        self.x = 10
-##        #print 'DEBUG Thing ', now(),self.M.lastTime
-##        self.M.accum(self.x,now())
-##        #print 'DEBUG Thing ', now(),self.M.lastTime
-##        yield hold,self,10.0
-##        #print 'DEBUG Thing ', now(),self.M.lastTime
-##        self.M.accum(self.x,now())
-##        #print 'DEBUG Thing ', now(),self.M.lastTime
-##
-##class makeMtestCase(unittest.TestCase):
-##    """ Test Monitor
-##    """
-##        
-##    def testTally(self):
-##        """Test Monitor - tally"""
-##        ## print 'Monitor version '+__version__
-##        m = Monitor(name='First')
-##        for i in range(10):
-##            m.tally(i)
-##        assert m.name == 'First','Tally name wrong'
-##        assert m.n == 10,'Tally count wrong'
-##        assert m.total == 45,'Tally total wrong'
-##        assert m.ttotal == 285,'Tally sum of squares wrong'
-##        assert m.count() == 10,'Tally count() wrong'
-##        assert m.mean() == 4.5,'Tally mean wrong'
-##        assert m.var() == (285-(45*45/10.0))/10.0,'Tally sample var wrong%s'%(m.var(),)
-##        assert str(m) == "Monitor First n=10 total= 45.0000",\
-##        'Tally monitor string wrong = '+str(m)
-##        ## test reset
-##        m.reset()
-##        assert m.n == 0,'Tally reset count wrong'
-##        assert m.total == 0.0,'Tally reset total wrong'
-##        assert m.ttotal == 0.0,'Tally reset sum of squares wrong'
-##        
-##
-##    def testAccumulate(self):
-##        """Test Monitor - accumulate"""
-##        #print 'Monitor version '+__version__
-##        m2 = Monitor(name='Second')
-##        assert m2.Stx == 0.0,'Accumulate Sum tx  wrong'
-##        assert m2.oldx == 0.0,'Accumulate oldx wrong'
-##        assert m2.lastTime == 0,'Accumulate lastTime wrong'
-##        assert m2.startTime == 0,'Accumulate startTime wrong'
-##        for i in range(10):
-##            m2.accum(10,i)
-##        #print m2
-##        assert m2.n == 10,'Accumulate count wrong'
-##        assert m2.total == 0.0,'Accumulate total wrong'
-##        assert m2.ttotal == 0.0,'Accumulate sum of squares wrong'
-##        assert m2.Stx == 90,'Accumulate sum tx wrong. It is %s'%(m2.Stx,)
-##        assert m2.lastTime == 9,'Accumulate lastTime wrong'
-##        assert m2.startTime == 0,'Accumulate startTime wrong'
-##        assert m2.count() == 10,'Accumulate count wrong. It is %s'%(m2.count(),)
-##        assert m2.timeAverage(9) == 10.0,'Accumulate timeAverage wrong. It is %s'%(m2.Stx,)
-##        assert str(m2) == "Monitor Second n=10 total=  0.0000 Stx =90.0000",\
-##        'Accumulate Monitor string wrong = '+str(m2)
-##        ## test reset
-##        m2.reset(10)
-##        assert m2.n == 0,'Accumulate reset count wrong'
-##        assert m2.total == 0.0,'Accumulate reset total wrong'
-##        assert m2.ttotal == 0.0,'Accumulate reset sum of squares wrong'
-##        assert m2.Stx == 0.0,'Accumulate reset sum tx wrong. It is %s'%(m2.Stx,)
-##        assert m2.lastTime == 10,'Accumulate lastTime wrong'
-##        assert m2.startTime == 10,'Accumulate startTime wrong'
-##
-##    def testAccumulateInTime(self):
-##        """Test Monitor - accumulate over simulation time"""
-##        #print 'Monitor version '+__version__
-##        initialize()
-##        m3 = Monitor(name='third')
-##        T3 = Thing(name="Job",M=m3)
-##        assert m3.Stx == 0.0,'Accumulate Sum tx  wrong'
-##        assert m3.oldx == 0.0,'Accumulate oldx wrong'
-##        assert m3.lastTime == 0,'Accumulate lastTime wrong'
-##        assert m3.startTime == 0,'Accumulate startTime wrong'
-##        activate(T3,T3.execute(),0.0)
-##        simulate(until=30.0)
-##        #print m3
-##        assert m3.startTime == 0,'Accumulate startTime wrong'
-##        assert m3.n == 2,'Accumulate n wrong'
-##        assert m3.oldx == 10.0,'Accumulate oldx wrong'
-##        assert m3.lastTime == 20.0,'Accumulate lastTime wrong. It is %s'%(m3.lastTime,)
-##        assert m3.Stx == 100.0,'Accumulate Sum tx  wrong. It is %s'%(m3.Stx,)
-##        #initialize()
-##       
-##
-##def makeMSuite():
-##    suite = unittest.TestSuite()
-##    testTally = makeMtestCase("testTally")
-##    testAccumulate = makeMtestCase("testAccumulate")
-##    testAccumulateInTime = makeMtestCase("testAccumulateInTime")
-##    suite.addTests([testTally,testAccumulate,testAccumulateInTime]) 
-##    return suite
-
 ##=====================================================
 ##                   Test Interrupts
 ##=====================================================
@@ -787,7 +712,9 @@ def makePSuite():
    return suite 
 
 if __name__ == '__main__':
-    alltests = unittest.TestSuite((makeSSuite(),makeRSuite(),makeMSuite(),makeHSuite(),makeISuite(),makePSuite()))
+    alltests = unittest.TestSuite((makeSSuite(),makeRSuite(),
+                                   makeMSuite(),makeHSuite(),
+                                   makeISuite(),makePSuite()))
     runner = unittest.TextTestRunner()
     runner.run(alltests)
 
